@@ -26,6 +26,7 @@
 #include "gdkcontentformats.h"
 #include "gdkcontentserializer.h"
 #include "gdkcontentdeserializer.h"
+#include "gdkdebugprivate.h"
 
 #include <gio/gio.h>
 
@@ -97,7 +98,7 @@ add_files (GDBusProxy  *proxy,
            AddFileData *afd)
 {
   GUnixFDList *fd_list;
-  GVariantBuilder fds;
+  GVariantBuilder fds, options;
   int i;
   char *key;
 
@@ -146,9 +147,10 @@ add_files (GDBusProxy  *proxy,
 
   key = (char *)g_object_get_data (G_OBJECT (afd->task), "key");
 
+  g_variant_builder_init (&options, G_VARIANT_TYPE_VARDICT);
   g_dbus_proxy_call_with_unix_fd_list (proxy,
                                        "AddFiles",
-                                       g_variant_new ("(sah)", key, &fds),
+                                       g_variant_new ("(saha{sv})", key, &fds, &options),
                                        0, -1,
                                        fd_list,
                                        NULL,
@@ -212,6 +214,8 @@ file_transfer_portal_register_files (const char          **files,
   afd->len = g_strv_length ((char **)files);
   afd->start = 0;
 
+  GDK_DEBUG (DND, "file transfer portal: registering %d files", afd->len);
+
   g_variant_builder_init (&options, G_VARIANT_TYPE_VARDICT);
   g_variant_builder_add (&options, "{sv}", "writable", g_variant_new_boolean (writable));
   g_variant_builder_add (&options, "{sv}", "autostop", g_variant_new_boolean (TRUE));
@@ -347,9 +351,8 @@ portal_ready (GObject *object,
 static void
 portal_file_serializer (GdkContentSerializer *serializer)
 {
-  GFile *file;
-  const GValue *value;
   GPtrArray *files;
+  const GValue *value;
 
   files = g_ptr_array_new_with_free_func (g_free);
 
@@ -357,9 +360,24 @@ portal_file_serializer (GdkContentSerializer *serializer)
 
   if (G_VALUE_HOLDS (value, G_TYPE_FILE))
     {
+      GFile *file;
+
       file = g_value_get_object (gdk_content_serializer_get_value (serializer));
-      if (file)
-        g_ptr_array_add (files, g_file_get_path (file));
+
+      if (file && g_file_peek_path (file))
+        {
+          GDK_DEBUG (DND, "file transfer portal: Adding %s", g_file_peek_path (file));
+          g_ptr_array_add (files, g_file_get_path (file));
+        }
+#ifdef G_ENABLE_DEBUG
+      else if (GDK_DEBUG_CHECK (DND))
+        {
+          char *uri = g_file_get_uri (file);
+          gdk_debug_message ("file transfer portal: %s has no path, dropping\n", uri);
+          g_free (uri);
+        }
+#endif
+
       g_ptr_array_add (files, NULL);
     }
   else if (G_VALUE_HOLDS (value, GDK_TYPE_FILE_LIST))
@@ -367,7 +385,21 @@ portal_file_serializer (GdkContentSerializer *serializer)
       GSList *l;
 
       for (l = g_value_get_boxed (value); l; l = l->next)
-        g_ptr_array_add (files, g_file_get_path (l->data));
+        {
+          GFile *file = l->data;
+
+          if (file && g_file_peek_path (file))
+            {
+              GDK_DEBUG (DND, "file transfer portal: Adding %s", g_file_peek_path (file));
+              g_ptr_array_add (files, g_file_get_path (file));
+            }
+          else
+            {
+              char *uri = g_file_get_uri (file);
+              gdk_debug_message ("file transfer portal: %s has no path, dropping", uri);
+              g_free (uri);
+            }
+        }
 
       g_ptr_array_add (files, NULL);
     }
@@ -394,6 +426,15 @@ portal_finish (GObject *object,
       return;
     }
 
+#ifdef G_ENABLE_DEBUG
+  if (GDK_DEBUG_CHECK (DND))
+    {
+      char *s = g_strjoinv (", ", files);
+      gdk_debug_message ("file transfer portal: Receiving files: %s", s);
+      g_free (s);
+    }
+#endif
+
   value = gdk_content_deserializer_get_value (deserializer);
   if (G_VALUE_HOLDS (value, G_TYPE_FILE))
     {
@@ -480,20 +521,36 @@ connection_closed (GDBusConnection *connection,
 }
 
 static void
-got_proxy (GObject *source,
-           GAsyncResult *result,
-           gpointer data)
+finish_registration (void)
 {
-  GError *error = NULL;
+  gdk_content_register_serializer (G_TYPE_FILE,
+                                   "application/vnd.portal.filetransfer",
+                                   portal_file_serializer,
+                                   NULL,
+                                   NULL);
 
-  file_transfer_proxy = g_dbus_proxy_new_for_bus_finish (result, &error);
-  if (!file_transfer_proxy)
-    {
-      g_message ("Failed to get file transfer portal: %s", error->message);
-      g_clear_error (&error);
-      return;
-    }
+  gdk_content_register_serializer (GDK_TYPE_FILE_LIST,
+                                   "application/vnd.portal.filetransfer",
+                                   portal_file_serializer,
+                                   NULL,
+                                   NULL);
 
+  gdk_content_register_deserializer ("application/vnd.portal.filetransfer",
+                                     GDK_TYPE_FILE_LIST,
+                                     portal_file_deserializer,
+                                     NULL,
+                                     NULL);
+
+  gdk_content_register_deserializer ("application/vnd.portal.filetransfer",
+                                     G_TYPE_FILE,
+                                     portal_file_deserializer,
+                                     NULL,
+                                     NULL);
+
+  /* FIXME: I missed up and used the wrong mime type here when
+   * I implemented my own protocol. Keep these around for a while
+   * so we can interoperate with existing flatpaks using GTK 4.6
+   */
   gdk_content_register_serializer (G_TYPE_FILE,
                                    "application/vnd.portal.files",
                                    portal_file_serializer,
@@ -523,6 +580,21 @@ got_proxy (GObject *source,
                     "closed", G_CALLBACK (connection_closed), NULL);
 }
 
+static gboolean
+proxy_has_owner (GDBusProxy *proxy)
+{
+  char *owner;
+
+  owner = g_dbus_proxy_get_name_owner (proxy);
+  if (owner)
+    {
+      g_free (owner);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 void
 file_transfer_portal_register (void)
 {
@@ -531,7 +603,8 @@ file_transfer_portal_register (void)
   if (!called)
     {
       called = TRUE;
-      g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+
+      file_transfer_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                 G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES
                                 | G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS
                                 | G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
@@ -540,8 +613,13 @@ file_transfer_portal_register (void)
                                 "/org/freedesktop/portal/documents",
                                 "org.freedesktop.portal.FileTransfer",
                                 NULL,
-                                got_proxy,
                                 NULL);
+
+      if (file_transfer_proxy && !proxy_has_owner (file_transfer_proxy))
+        g_clear_object (&file_transfer_proxy);
+
+      if (file_transfer_proxy)
+        finish_registration ();
     }
 }
 
