@@ -423,7 +423,7 @@ static const GdkMemoryFormatDescription memory_formats[] = {
     G_ALIGNOF (guchar),
     GDK_MEMORY_U8,
     { 0, 0, 0, 0 },
-    { GL_RGB8, GL_RGB, GL_UNSIGNED_BYTE, { GL_RED, GL_GREEN, GL_BLUE, GL_ONE } },
+    { GL_RGB8, GL_RGB, GL_UNSIGNED_BYTE, { GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA } },
     r8g8b8_to_float,
     r8g8b8_from_float,
   },
@@ -433,7 +433,7 @@ static const GdkMemoryFormatDescription memory_formats[] = {
     G_ALIGNOF (guchar),
     GDK_MEMORY_U8,
     { 0, 0, G_MAXUINT, G_MAXUINT },
-    { GL_RGB8, GL_BGR, GL_UNSIGNED_BYTE, { GL_RED, GL_GREEN, GL_BLUE, GL_ONE } },
+    { GL_RGB8, GL_BGR, GL_UNSIGNED_BYTE, { GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA } },
     b8g8r8_to_float,
     b8g8r8_from_float,
   },
@@ -443,7 +443,7 @@ static const GdkMemoryFormatDescription memory_formats[] = {
     G_ALIGNOF (guint16),
     GDK_MEMORY_U16,
     { 0, 0, 3, 0 },
-    { GL_RGB16, GL_RGB, GL_UNSIGNED_SHORT, { GL_RED, GL_GREEN, GL_BLUE, GL_ONE } },
+    { GL_RGB16, GL_RGB, GL_UNSIGNED_SHORT, { GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA } },
     r16g16b16_to_float,
     r16g16b16_from_float,
   },
@@ -473,7 +473,7 @@ static const GdkMemoryFormatDescription memory_formats[] = {
     G_ALIGNOF (guint16),
     GDK_MEMORY_FLOAT16,
     { 0, 0, 3, 0 },
-    { GL_RGB16F, GL_RGB, GL_HALF_FLOAT, { GL_RED, GL_GREEN, GL_BLUE, GL_ONE } },
+    { GL_RGB16F, GL_RGB, GL_HALF_FLOAT, { GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA } },
     r16g16b16_float_to_float,
     r16g16b16_float_from_float,
   },
@@ -503,7 +503,7 @@ static const GdkMemoryFormatDescription memory_formats[] = {
     G_ALIGNOF (float),
     GDK_MEMORY_FLOAT32,
     { 0, 0, 3, 0 },
-    { GL_RGB32F, GL_RGB, GL_FLOAT, { GL_RED, GL_GREEN, GL_BLUE, GL_ONE } },
+    { GL_RGB32F, GL_RGB, GL_FLOAT, { GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA } },
     r32g32b32_float_to_float,
     r32g32b32_float_from_float,
   },
@@ -511,7 +511,7 @@ static const GdkMemoryFormatDescription memory_formats[] = {
     GDK_MEMORY_ALPHA_PREMULTIPLIED,
     16,
     G_ALIGNOF (float),
-    TRUE,
+    GDK_MEMORY_FLOAT32,
     { 0, 0, 3, 0 },
     { GL_RGBA32F, GL_RGBA, GL_FLOAT, { GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA } },
     r32g32b32a32_float_to_float,
@@ -739,12 +739,12 @@ gdk_memory_format_gl_format (GdkMemoryFormat  format,
                              guint           *out_internal_format,
                              guint           *out_format,
                              guint           *out_type,
-                             GLint          (*out_swizzle)[4])
+                             GLint            out_swizzle[4])
 {
   *out_internal_format = memory_formats[format].gl.internal_format;
   *out_format = memory_formats[format].gl.format;
   *out_type = memory_formats[format].gl.type;
-  memcpy (out_swizzle, &memory_formats[format].gl.swizzle, sizeof(GLint) * 4);
+  memcpy (out_swizzle, memory_formats[format].gl.swizzle, sizeof(GLint) * 4);
 
   if (gles)
     {
