@@ -17,8 +17,7 @@
 
 /* Uninstalled header defining types and functions internal to GDK */
 
-#ifndef __GDK_SURFACE_PRIVATE_H__
-#define __GDK_SURFACE_PRIVATE_H__
+#pragma once
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include "gdkenumtypes.h"
@@ -27,13 +26,6 @@
 
 G_BEGIN_DECLS
 
-typedef enum
-{
-  GDK_SURFACE_TOPLEVEL,
-  GDK_SURFACE_POPUP,
-  GDK_SURFACE_DRAG
-} GdkSurfaceType;
-
 struct _GdkSurface
 {
   GObject parent_instance;
@@ -56,10 +48,6 @@ struct _GdkSurface
   cairo_region_t *update_area;
   guint update_freeze_count;
   GdkFrameClockPhase pending_phases;
-  /* This is the update_area that was in effect when the current expose
-     started. It may be smaller than the expose area if we'e painting
-     more than we have to, but it represents the "true" damage. */
-  cairo_region_t *active_update_area;
 
   GdkToplevelState pending_set_flags;
   GdkToplevelState pending_unset_flags;
@@ -72,7 +60,6 @@ struct _GdkSurface
 
   guint modal_hint : 1;
   guint destroyed : 2;
-  guint in_update : 1;
   guint frame_clock_events_paused : 1;
   guint autohide : 1;
   guint shortcuts_inhibited : 1;
@@ -111,8 +98,6 @@ struct _GdkSurfaceClass
 {
   GObjectClass parent_class;
 
-  cairo_surface_t *
-               (* ref_cairo_surface)    (GdkSurface      *surface);
   void         (* hide)                 (GdkSurface      *surface);
   void         (* get_geometry)         (GdkSurface      *surface,
                                          int             *x,
@@ -155,7 +140,7 @@ struct _GdkSurfaceClass
                                          double              dx,
                                          double              dy);
 
-  int          (* get_scale_factor)       (GdkSurface      *surface);
+  double       (* get_scale)              (GdkSurface      *surface);
 
   void         (* set_opaque_region)      (GdkSurface      *surface,
                                            cairo_region_t *region);
@@ -293,6 +278,8 @@ void gdk_surface_get_geometry (GdkSurface *surface,
                                int        *width,
                                int        *height);
 
+void                    gdk_surface_set_frame_clock             (GdkSurface             *surface,
+                                                                 GdkFrameClock          *clock);
 void                    gdk_surface_set_egl_native_window       (GdkSurface             *self,
                                                                  gpointer                native_window);
 void                    gdk_surface_ensure_egl_surface          (GdkSurface             *self,
@@ -351,4 +338,3 @@ gboolean       gdk_surface_supports_edge_constraints    (GdkSurface *surface);
 
 G_END_DECLS
 
-#endif /* __GDK_SURFACE_PRIVATE_H__ */
