@@ -83,7 +83,7 @@ typedef struct
 
   GskDebugFlags debug_flags;
 
-  gboolean is_realized : 1;
+  unsigned int is_realized : 1;
 } GskRendererPrivate;
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GskRenderer, gsk_renderer, G_TYPE_OBJECT)
@@ -238,24 +238,6 @@ gsk_renderer_get_surface (GskRenderer *renderer)
   return priv->surface;
 }
 
-/*< private >
- * gsk_renderer_get_root_node:
- * @renderer: a `GskRenderer`
- *
- * Retrieves the `GskRenderNode` used by @renderer.
- *
- * Returns: (transfer none) (nullable): a `GskRenderNode`
- */
-GskRenderNode *
-gsk_renderer_get_root_node (GskRenderer *renderer)
-{
-  GskRendererPrivate *priv = gsk_renderer_get_instance_private (renderer);
-
-  g_return_val_if_fail (GSK_IS_RENDERER (renderer), NULL);
-
-  return priv->root_node;
-}
-
 /**
  * gsk_renderer_is_realized: (attributes org.gtk.Method.get_property=realized)
  * @renderer: a `GskRenderer`
@@ -313,6 +295,11 @@ gsk_renderer_realize (GskRenderer  *renderer,
     }
 
   priv->is_realized = TRUE;
+
+  g_object_notify (G_OBJECT (renderer), "realized");
+  if (surface)
+    g_object_notify (G_OBJECT (renderer), "surface");
+
   return TRUE;
 }
 
@@ -326,17 +313,25 @@ void
 gsk_renderer_unrealize (GskRenderer *renderer)
 {
   GskRendererPrivate *priv = gsk_renderer_get_instance_private (renderer);
+  gboolean has_surface;
 
   g_return_if_fail (GSK_IS_RENDERER (renderer));
 
   if (!priv->is_realized)
     return;
 
+  has_surface = priv->surface != NULL;
+
   GSK_RENDERER_GET_CLASS (renderer)->unrealize (renderer);
 
+  g_clear_object (&priv->surface);
   g_clear_pointer (&priv->prev_node, gsk_render_node_unref);
 
   priv->is_realized = FALSE;
+
+  g_object_notify (G_OBJECT (renderer), "realized");
+  if (has_surface)
+    g_object_notify (G_OBJECT (renderer), "surface");
 }
 
 /**
