@@ -3,6 +3,8 @@
 #include "gskrendernode.h"
 #include <cairo.h>
 
+#include "gdk/gdkmemoryformatprivate.h"
+
 G_BEGIN_DECLS
 
 typedef struct _GskRenderNodeClass GskRenderNodeClass;
@@ -19,6 +21,9 @@ extern GType gsk_render_node_types[];
 #define GSK_IS_RENDER_NODE_TYPE(node,type) \
   (G_TYPE_INSTANCE_GET_CLASS ((node), GSK_TYPE_RENDER_NODE, GskRenderNodeClass)->node_type == (type))
 
+#define GSK_RENDER_NODE_TYPE(node) \
+  (G_TYPE_INSTANCE_GET_CLASS ((node), GSK_TYPE_RENDER_NODE, GskRenderNodeClass)->node_type)
+
 struct _GskRenderNode
 {
   GTypeInstance parent_instance;
@@ -27,7 +32,7 @@ struct _GskRenderNode
 
   graphene_rect_t bounds;
 
-  guint prefers_high_depth : 1;
+  guint preferred_depth : 2;
   guint offscreen_for_opacity : 1;
 };
 
@@ -55,6 +60,8 @@ GType           gsk_render_node_type_register_static    (const char
 
 gpointer        gsk_render_node_alloc                   (GskRenderNodeType            node_type);
 
+void            _gsk_render_node_unref                  (GskRenderNode               *node);
+
 gboolean        gsk_render_node_can_diff                (const GskRenderNode         *node1,
                                                          const GskRenderNode         *node2) G_GNUC_PURE;
 void            gsk_render_node_diff                    (GskRenderNode               *node1,
@@ -73,18 +80,27 @@ bool            gsk_border_node_get_uniform_color       (const GskRenderNode
 void            gsk_text_node_serialize_glyphs          (GskRenderNode               *self,
                                                          GString                     *str);
 
-GskRenderNode ** gsk_container_node_get_children        (const GskRenderNode *node,
-                                                         guint               *n_children);
+GskRenderNode ** gsk_container_node_get_children        (const GskRenderNode         *node,
+                                                         guint                       *n_children);
 
-void             gsk_transform_node_get_translate       (const GskRenderNode *node,
-                                                         float               *dx,
-                                                         float               *dy);
-gboolean       gsk_render_node_prefers_high_depth       (const GskRenderNode *node);
+void            gsk_transform_node_get_translate        (const GskRenderNode         *node,
+                                                         float                       *dx,
+                                                         float                       *dy);
+GdkMemoryDepth  gsk_render_node_get_preferred_depth     (const GskRenderNode         *node);
 
-gboolean       gsk_container_node_is_disjoint           (const GskRenderNode *node);
+gboolean        gsk_container_node_is_disjoint          (const GskRenderNode         *node);
 
-gboolean       gsk_render_node_use_offscreen_for_opacity (const GskRenderNode *node);
+gboolean        gsk_render_node_use_offscreen_for_opacity (const GskRenderNode       *node);
 
+#define gsk_render_node_ref(node)   _gsk_render_node_ref(node)
+#define gsk_render_node_unref(node) _gsk_render_node_unref(node)
+
+static inline GskRenderNode *
+_gsk_render_node_ref (GskRenderNode *node)
+{
+  g_atomic_ref_count_inc (&node->ref_count);
+  return node;
+}
 
 G_END_DECLS
 
