@@ -27,7 +27,6 @@
 #include "gtkadjustmentprivate.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
-#include "gtkintl.h"
 
 
 /**
@@ -35,7 +34,7 @@
  *
  * `GtkAdjustment` is a model for a numeric value.
  *
- * The `GtkAdjustment has an associated lower and upper bound.
+ * The `GtkAdjustment` has an associated lower and upper bound.
  * It also contains step and page increments, and a page size.
  *
  * Adjustments are used within several GTK widgets, including
@@ -353,6 +352,21 @@ gtk_adjustment_dispatch_properties_changed (GObject     *object,
     }
 }
 
+static double
+gtk_adjustment_sanitize_value (GtkAdjustment *self,
+                               double         value)
+{
+  GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (self);
+
+  /* don't use CLAMP() so we don't end up below lower if upper - page_size
+   * is smaller than lower
+   */
+  value = MIN (value, priv->upper - priv->page_size);
+  value = MAX (value, priv->lower);
+
+  return value;
+}
+
 /**
  * gtk_adjustment_new:
  * @value: the initial value
@@ -420,6 +434,8 @@ adjustment_set_value (GtkAdjustment *adjustment,
 {
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
+  value = gtk_adjustment_sanitize_value (adjustment, value);
+
   if (priv->value != value)
     {
       priv->value = value;
@@ -498,11 +514,7 @@ gtk_adjustment_set_value_internal (GtkAdjustment *adjustment,
 {
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
-  /* don't use CLAMP() so we don't end up below lower if upper - page_size
-   * is smaller than lower
-   */
-  value = MIN (value, priv->upper - priv->page_size);
-  value = MAX (value, priv->lower);
+  value = gtk_adjustment_sanitize_value (adjustment, value);
 
   if (animate && priv->duration != 0 && priv->clock != NULL)
     {
@@ -826,11 +838,7 @@ gtk_adjustment_configure (GtkAdjustment *adjustment,
   gtk_adjustment_set_page_increment (adjustment, page_increment);
   gtk_adjustment_set_page_size (adjustment, page_size);
 
-  /* don't use CLAMP() so we don't end up below lower if upper - page_size
-   * is smaller than lower
-   */
-  value = MIN (value, upper - page_size);
-  value = MAX (value, lower);
+  value = gtk_adjustment_sanitize_value (adjustment, value);
 
   if (value != priv->value)
     {
