@@ -17,13 +17,14 @@
  * Authors: Benjamin Otte <otte@gnome.org>
  */
 
-#ifndef __GTK_CSS_IMAGE_PRIVATE_H__
-#define __GTK_CSS_IMAGE_PRIVATE_H__
+#pragma once
 
 #include <cairo.h>
 #include <glib-object.h>
 
-#include "gtk/gtkcssparserprivate.h"
+#include <gtk/css/gtkcss.h>
+#include "gtk/css/gtkcsstokenizerprivate.h"
+#include "gtk/css/gtkcssparserprivate.h"
 #include "gtk/gtkcsstypesprivate.h"
 #include "gtk/gtksnapshot.h"
 #include "gtk/gtkstyleprovider.h"
@@ -146,4 +147,3 @@ gboolean       gtk_css_image_is_computed           (GtkCssImage                *
 
 G_END_DECLS
 
-#endif /* __GTK_CSS_IMAGE_PRIVATE_H__ */
