@@ -54,7 +54,8 @@
  *
  * ![An example GtkDropDown](drop-down.png)
  *
- * The `GtkDropDown` displays the selected choice.
+ * The `GtkDropDown` displays the [selected][property@Gtk.DropDown:selected]
+ * choice.
  *
  * The options are given to `GtkDropDown` in the form of `GListModel`
  * and how the individual options are represented is determined by
@@ -74,6 +75,7 @@
  * use [method@Gtk.DropDown.set_enable_search].
  *
  * Here is a UI definition example for `GtkDropDown` with a simple model:
+ *
  * ```xml
  * <object class="GtkDropDown">
  *   <property name="model">
@@ -91,12 +93,12 @@
  * To learn more about the list widget framework, see the
  * [overview](section-list-widget.html).
  *
- * # CSS nodes
+ * ## CSS nodes
  *
  * `GtkDropDown` has a single CSS node with name dropdown,
  * with the button and popover nodes as children.
  *
- * # Accessibility
+ * ## Accessibility
  *
  * `GtkDropDown` uses the %GTK_ACCESSIBLE_ROLE_COMBO_BOX role.
  */
@@ -127,7 +129,7 @@ struct _GtkDropDown
   GtkWidget *search_entry;
 
   GtkExpression *expression;
-  
+
   GtkStringFilterMatchMode search_match_mode;
 
   guint enable_search : 1;
@@ -394,7 +396,7 @@ gtk_drop_down_get_property (GObject    *object,
     case PROP_SHOW_ARROW:
       g_value_set_boolean (value, gtk_drop_down_get_show_arrow (self));
       break;
-      
+
     case PROP_SEARCH_MATCH_MODE:
       g_value_set_enum (value, gtk_drop_down_get_search_match_mode (self));
       break;
@@ -446,7 +448,7 @@ gtk_drop_down_set_property (GObject      *object,
     case PROP_SHOW_ARROW:
       gtk_drop_down_set_show_arrow (self, g_value_get_boolean (value));
       break;
-      
+
     case PROP_SEARCH_MATCH_MODE:
       gtk_drop_down_set_search_match_mode (self, g_value_get_enum (value));
       break;
@@ -605,8 +607,8 @@ gtk_drop_down_class_init (GtkDropDownClass *klass)
    */
   properties[PROP_SELECTED] =
     g_param_spec_uint ("selected", NULL, NULL,
-                         0, G_MAXUINT, GTK_INVALID_LIST_POSITION,
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+                       0, G_MAXUINT, GTK_INVALID_LIST_POSITION,
+                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GtkDropDown:selected-item: (attributes org.gtk.Property.get=gtk_drop_down_get_selected_item)
@@ -615,8 +617,8 @@ gtk_drop_down_class_init (GtkDropDownClass *klass)
    */
   properties[PROP_SELECTED_ITEM] =
     g_param_spec_object ("selected-item", NULL, NULL,
-                       G_TYPE_OBJECT,
-                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+                         G_TYPE_OBJECT,
+                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
    * GtkDropDown:enable-search: (attributes org.gtk.Property.get=gtk_drop_down_get_enable_search org.gtk.Property.set=gtk_drop_down_set_enable_search)
@@ -656,7 +658,7 @@ gtk_drop_down_class_init (GtkDropDownClass *klass)
     g_param_spec_boolean  ("show-arrow", NULL, NULL,
                            TRUE,
                            G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
-                           
+
   /**
    * GtkDropDown:search-match-mode: (attributes org.gtk.Property.get=gtk_drop_down_get_search_match_mode org.gtk.Property.set=gtk_drop_down_set_search_match_mode)
    *
@@ -669,7 +671,7 @@ gtk_drop_down_class_init (GtkDropDownClass *klass)
                            GTK_TYPE_STRING_FILTER_MATCH_MODE,
                            GTK_STRING_FILTER_MATCH_MODE_PREFIX,
                            G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
-                           
+
   g_object_class_install_properties (gobject_class, N_PROPS, properties);
 
   /**
@@ -752,6 +754,21 @@ selected_item_changed (GtkDropDown *self,
     gtk_widget_set_opacity (icon, 0.0);
 }
 
+static void
+root_changed (GtkWidget   *box,
+              GParamSpec  *pspec,
+              GtkDropDown *self)
+{
+  GtkWidget *icon;
+
+  icon = gtk_widget_get_last_child (box);
+
+  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == self->popup)
+    gtk_widget_set_visible (icon, TRUE);
+  else
+    gtk_widget_set_visible (icon, FALSE);
+}
+
 static void
 bind_item (GtkSignalListItemFactory *factory,
            GtkListItem              *list_item,
@@ -761,13 +778,11 @@ bind_item (GtkSignalListItemFactory *factory,
   gpointer item;
   GtkWidget *box;
   GtkWidget *label;
-  GtkWidget *icon;
   GValue value = G_VALUE_INIT;
 
   item = gtk_list_item_get_item (list_item);
   box = gtk_list_item_get_child (list_item);
   label = gtk_widget_get_first_child (box);
-  icon = gtk_widget_get_last_child (box);
 
   if (self->expression &&
       gtk_expression_evaluate (self->expression, item, &value))
@@ -787,17 +802,13 @@ bind_item (GtkSignalListItemFactory *factory,
       g_critical ("Either GtkDropDown:factory or GtkDropDown:expression must be set");
     }
 
-  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == self->popup)
-    {
-      gtk_widget_set_visible (icon, TRUE);
-      g_signal_connect (self, "notify::selected-item",
-                        G_CALLBACK (selected_item_changed), list_item);
-      selected_item_changed (self, NULL, list_item);
-    }
-  else
-    {
-      gtk_widget_set_visible (icon, FALSE);
-    }
+  g_signal_connect (self, "notify::selected-item",
+                    G_CALLBACK (selected_item_changed), list_item);
+  selected_item_changed (self, NULL, list_item);
+
+  g_signal_connect (box, "notify::root",
+                    G_CALLBACK (root_changed), self);
+  root_changed (box, NULL, self);
 }
 
 static void
@@ -806,8 +817,12 @@ unbind_item (GtkSignalListItemFactory *factory,
              gpointer                  data)
 {
   GtkDropDown *self = data;
+  GtkWidget *box;
+
+  box = gtk_list_item_get_child (list_item);
 
   g_signal_handlers_disconnect_by_func (self, selected_item_changed, list_item);
+  g_signal_handlers_disconnect_by_func (box, root_changed, self);
 }
 
 static void
@@ -841,7 +856,7 @@ gtk_drop_down_init (GtkDropDown *self)
   self->show_arrow = gtk_widget_get_visible (self->arrow);
 
   set_default_factory (self);
- 
+
   self->search_match_mode = GTK_STRING_FILTER_MATCH_MODE_PREFIX;
 }
 
@@ -1311,7 +1326,7 @@ gtk_drop_down_get_show_arrow (GtkDropDown *self)
  * @search_match_mode: the new match mode
  *
  * Sets the match mode for the search filter.
- * 
+ *
  * Since: 4.12
  */
 void
@@ -1337,7 +1352,7 @@ gtk_drop_down_set_search_match_mode (GtkDropDown *self,
  * Returns the match mode that the search filter is using.
  *
  * Returns: the match mode of the search filter
- * 
+ *
  * Since: 4.12
  */
 GtkStringFilterMatchMode
