@@ -18,7 +18,6 @@
 #include "config.h"
 
 #include "gtkentrybuffer.h"
-#include "gtkintl.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtkwidget.h"
@@ -389,6 +388,9 @@ gtk_entry_buffer_class_init (GtkEntryBufferClass *klass)
                                          G_TYPE_UINT,
                                          G_TYPE_STRING,
                                          G_TYPE_UINT);
+  g_signal_set_va_marshaller (signals[INSERTED_TEXT],
+                              GTK_TYPE_ENTRY_BUFFER,
+                              _gtk_marshal_VOID__UINT_STRING_UINTv);
 
   /**
    * GtkEntryBuffer::deleted-text:
@@ -410,6 +412,9 @@ gtk_entry_buffer_class_init (GtkEntryBufferClass *klass)
                                          G_TYPE_NONE, 2,
                                          G_TYPE_UINT,
                                          G_TYPE_UINT);
+  g_signal_set_va_marshaller (signals[DELETED_TEXT],
+                              GTK_TYPE_ENTRY_BUFFER,
+                              _gtk_marshal_VOID__UINT_UINTv);
 }
 
 /* --------------------------------------------------------------------------------
