@@ -708,6 +708,16 @@ gtk_snapshot_collect_repeat (GtkSnapshot      *snapshot,
   return repeat_node;
 }
 
+static GskRenderNode *
+gtk_snapshot_collect_discard_repeat (GtkSnapshot      *snapshot,
+                                     GtkSnapshotState *state,
+                                     GskRenderNode   **nodes,
+                                     guint             n_nodes)
+{
+  /* Drop the node and return nothing.  */
+  return NULL;
+}
+
 static void
 gtk_graphene_rect_scale_affine (const graphene_rect_t *rect,
                                 float                  scale_x,
@@ -798,17 +808,24 @@ gtk_snapshot_push_repeat (GtkSnapshot           *snapshot,
                           const graphene_rect_t *child_bounds)
 {
   GtkSnapshotState *state;
+  gboolean empty_child_bounds = FALSE;
   graphene_rect_t real_child_bounds = { { 0 } };
   float scale_x, scale_y, dx, dy;
 
   gtk_snapshot_ensure_affine (snapshot, &scale_x, &scale_y, &dx, &dy);
 
   if (child_bounds)
-    gtk_graphene_rect_scale_affine (child_bounds, scale_x, scale_y, dx, dy, &real_child_bounds);
+    {
+      gtk_graphene_rect_scale_affine (child_bounds, scale_x, scale_y, dx, dy, &real_child_bounds);
+      if (real_child_bounds.size.width <= 0 || real_child_bounds.size.height <= 0)
+        empty_child_bounds = TRUE;
+    }
 
   state = gtk_snapshot_push_state (snapshot,
                                    gtk_snapshot_get_current_state (snapshot)->transform,
-                                   gtk_snapshot_collect_repeat,
+                                   empty_child_bounds
+                                   ? gtk_snapshot_collect_discard_repeat
+                                   : gtk_snapshot_collect_repeat,
                                    NULL);
 
   gtk_graphene_rect_scale_affine (bounds, scale_x, scale_y, dx, dy, &state->data.repeat.bounds);
@@ -1704,9 +1721,10 @@ gtk_snapshot_gl_shader_pop_texture (GtkSnapshot *snapshot)
  * on an internal stack.
  *
  * When [method@Gtk.Snapshot.restore] is called, @snapshot will
- * be restored to the saved state. Multiple calls to
- * [method@Snapshot.save] and [class@Snapshot.restore] can be nested;
- * each call to `gtk_snapshot_restore()` restores the state from
+ * be restored to the saved state.
+ *
+ * Multiple calls to [method@Gtk.Snapshot.save] and [method@Gtk.Snapshot.restore]
+ * can be nested; each call to `gtk_snapshot_restore()` restores the state from
  * the matching paired `gtk_snapshot_save()`.
  *
  * It is necessary to clear all saved states with corresponding
@@ -1841,9 +1859,10 @@ gtk_snapshot_translate_3d (GtkSnapshot              *snapshot,
  * @angle: the rotation angle, in degrees (clockwise)
  *
  * Rotates @@snapshot's coordinate system by @angle degrees in 2D space -
- * or in 3D speak, rotates around the Z axis.
+ * or in 3D speak, rotates around the Z axis. The rotation happens around
+ * the origin point of (0, 0) in the @snapshot's current coordinate system.
  *
- * To rotate around other axes, use [method@Gsk.Transform.rotate_3d].
+ * To rotate around axes other than the Z axis, use [method@Gsk.Transform.rotate_3d].
  */
 void
 gtk_snapshot_rotate (GtkSnapshot *snapshot,
