@@ -32,7 +32,6 @@
 #include "gtkstack.h"
 #include "gtkprivate.h"
 #include "gtkwidgetprivate.h"
-#include "gtkintl.h"
 
 #include <glib/gi18n-lib.h>
 
@@ -221,6 +220,12 @@ add_child (guint            position,
   row = gtk_list_box_row_new ();
   gtk_list_box_row_set_child (GTK_LIST_BOX_ROW (row), item);
 
+  gtk_accessible_update_relation (GTK_ACCESSIBLE (row),
+                                  GTK_ACCESSIBLE_RELATION_LABELLED_BY,
+                                  item,
+                                  NULL,
+                                  -1);
+
   page = g_list_model_get_item (G_LIST_MODEL (self->pages), position);
   update_row (self, page, row);
 
@@ -373,8 +378,7 @@ gtk_stack_sidebar_class_init (GtkStackSidebarClass *klass)
    * The stack.
    */
   obj_properties[PROP_STACK] =
-      g_param_spec_object (I_("stack"), P_("Stack"),
-                           P_("Associated stack for this GtkStackSidebar"),
+      g_param_spec_object (I_("stack"), NULL, NULL,
                            GTK_TYPE_STACK,
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
