@@ -376,10 +376,14 @@ row_data_update_info (RowData   *data,
       icon = g_file_icon_new (thumbnail_file);
       g_object_unref (thumbnail_file);
     }
-  else
+  else if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_ICON))
     {
       icon = g_file_info_get_icon (info);
     }
+  else
+    {
+      icon = NULL;
+    }
 
   gtk_widget_set_visible (data->icon, icon != NULL);
   gtk_image_set_from_gicon (GTK_IMAGE (data->icon), icon);
@@ -479,7 +483,7 @@ row_data_free (gpointer _data)
 
   row_data_unbind (data);
 
-  g_slice_free (RowData, data);
+  g_free (data);
 }
 
 static void
@@ -489,7 +493,7 @@ setup_widget (GtkSignalListItemFactory *factory,
   GtkWidget *box, *child;
   RowData *data;
 
-  data = g_slice_new0 (RowData);
+  data = g_new0 (RowData, 1);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
   gtk_list_item_set_child (list_item, box);
